# Go up by 2 directory and import 

import sys
import os.path as path
two_up =  path.abspath(path.join(__file__ ,"../.."))
sys.path.append(two_up)

import YW_matchups




# test Pleiades


sat_folder = '/Volumes/San2T/241027_Pleiades/ACed'



is_file = '/Volumes/San2T/241027_Pleiades/Matchups/lab_for_matchups.csv'
out_file = '/Volumes/San2T/241027_Pleiades/Matchups/Pleiades_lab.csv'


test = YW_matchups.run(sat_folder, is_file, AC='ACOLITE', out_file=out_file, dt_column='datetime',dt_string='%m/%d/%Y %H:%M')











'''
# Plot 

test = test[0]

from matplotlib import pyplot as plt
plt.imshow(test, interpolation='nearest')
plt.show()




plt.imshow(rhow_data[0], interpolation='nearest')
plt.show()


'''

