# Go up by 2 directory and import 

import sys

sys.path.append('/Users/yw/Desktop/YW_private_code/YW_matchups')

import YW_matchups


### in situ 
is_file = '/Volumes/San2T/240727_Chapter3/Matchups/spectral.csv'


dt_format = '%Y-%m-%d %H:%M'


# L89
sat_folder = '/Volumes/San2T/240727_Chapter3/ACed/L89'
out_file = '/Users/yw/Downloads/test/spectral_L89.csv'
test = YW_matchups.run(sat_folder, is_file, AC='ACOLITE', out_file=out_file, dt_column='datetime',
                       dt_string=dt_format, ACOLITE_L2R=True)
